# KnifeGrenade release 5 by David Bowland
# ./addons/eventscripts/knifegrenade/knifegrenade.py

# >>> To configure this addon please see knifegrenade.cfg <<<

"""
Allows players to pull the pin on the grenade of an opponent by attacking with a knife. Requires only ES 2.0+
"""


import es
import gamethread
import langlib
import os.path
import playerlib
import random


info = es.AddonInfo()
info.name = 'KnifeGrenade'
info.version = '5'
info.url = 'http://addons.eventscripts.com/addons/view/knifegrenade'
info.basename = 'knifegrenade'
info.author = 'SuperDave'


int_noisy = 0
dict_players = {}

dict_options = {'knifegrenade_pinpull_givegrenade':[1, '0 = no change, 1 = if the attacker has a HE grenade and the victim doesn\'t, the attacker\'s grenade will be given to the victim, 2 = always give the victim a grenade'], 'knifegrenade_pinpull_explodeondeath':[1, '0 = no change, 1 = players with a ticking grenade that die will explode when they die'], 'knifegrenade_detonate_delay':[4, 'Number of seconds after a pin pull a grenade explodes'], 'knifegrenade_detonate_radius':[350, 'Radius of explosion when a grenade with its pin pulled explodes--default HE grenade radius is 350'], 'knifegrenade_detonate_magnitude':[300, 'Radius of explosion when a grenade with its pin pulled explodes--default HE grenade magnitude is 100'], 'knifegrenade_sound_percentpinpull':[85, 'Percent of the map that can hear grenade pin pulls--set to 0 to eliminate'], 'knifegrenade_sound_percenttick':[60, 'Percent of the map that can hear grenade ticks--set to 0 to eliminate'], 'knifegrenade_sound_percentdetonate':[45, 'Percent of the map that can hear grenade explosions--set to 0 to eliminate'], 'knifegrenade_announce_victim':[0, '0 = no change, 1 = announce to player when the pin is pulled on his or her grenade'], 'knifegrenade_announce_attacker':[1, '0 = no change, 1 = announce to player when he or she pulls the pin on another player\'s grenade'], 'knifegrenade_announce_all':[2, '0 = no change, 1 = announce to all players when a player has a grenade pin pulled in chat area, 2 = announce to all players when a player has a grenade pin pulled on the console']}
func_lang_text = None


def load():
   """
   Checks to make sure critical server variables are created by the config
   Turns on eventscripts_noisy if necessary
   """
   global int_noisy
   global func_lang_text

   for str_option in dict_options:
      es.ServerVar(str_option, dict_options[str_option][0], dict_options[str_option][1])
   if os.path.isfile(es.getAddonPath('knifegrenade') + '/knifegrenade.cfg'):
      es.server.cmd('es_xmexec ../addons/eventscripts/knifegrenade/knifegrenade.cfg')
   else:
      es.dbgmsg(0, 'KnifeGrenade: Unable to load knifegrenade.cfg! Please ensure it is in the ./knifegrenade/ directory.')

   str_lang_path = es.getAddonPath('knifegrenade') + '/knifegrenade_languages.ini'
   if os.path.isfile(str_lang_path):
      func_lang_text = langlib.Strings(str_lang_path)
   else:
      es.dbgmsg(0, 'KnifeGrenade: Unable to load knifegrenade_languages.ini! Please ensure it is in the ./knifegrenade/ directory.')
   if not func_lang_text:
      func_lang_text = missing_lang

   es.doblock('corelib/noisy_on')
   if not int(es.ServerVar('eventscripts_noisy')):
      int_noisy = 1
      es.ServerVar('eventscripts_noisy').set(1)


def es_map_start(event_var):
   """Cancels outstanding delays"""
   for str_userid in dict_players:
      gamethread.cancelDelayed('knifegrenade_%s' % str_userid)
   dict_players.clear()


def player_spawn(event_var):
   """Removes the delay for the spawning player"""
   remove_delay(event_var['userid'])


def player_hurt(event_var):
   """
   Determines if the hurt player had a pin pulled
   Gives the player a HE grenade if necessary
   Announces pin pull
   Plays pin pull sound and starts tick delay
   """
   global dict_players

   if event_var['weapon'] == 'knife' and event_var['es_userteam'] <> event_var['es_attackerteam']:
      str_userid = event_var['userid']
      str_attackerid = event_var['attacker']
      if not dict_players.has_key(str_userid):
         int_grenade_index = 0
         int_player_handle = es.getplayerhandle(str_userid)
         for int_index in es.createentitylist('weapon_hegrenade'):
            if int_player_handle == es.getindexprop(int_index, 'CBaseEntity.m_hOwnerEntity'):
               int_grenade_index = int_index

         if not int_grenade_index:
            int_give_grenade = int(es.ServerVar('knifegrenade_pinpull_givegrenade'))
            if int_give_grenade == 2:
               es.server.cmd('es_xgive %s weapon_hegrenade' % str_userid)
               int_grenade_index = 1
            elif int_give_grenade:
               int_player_handle = es.getplayerhandle(str_attackerid)
               for int_index in es.createentitylist('weapon_hegrenade'):
                  if int_player_handle == es.getindexprop(int_index, 'CBaseEntity.m_hOwnerEntity'):
                     int_grenade_index = int_index
               if int_grenade_index:
                  es.server.cmd('es_xremove %s' % int_grenade_index)
                  es.server.cmd('es_xgive %s weapon_hegrenade' % str_userid)

         if int_grenade_index:
            dict_tokens = {'attacker':event_var['es_attackername'], 'victim':event_var['es_username']}
            if int(es.ServerVar('knifegrenade_announce_victim')):
               es.tell(str_userid, '#multi', func_lang_text('pin pulled', dict_tokens, playerlib.getPlayer(str_userid).get('lang')))
            if int(es.ServerVar('knifegrenade_announce_attacker')):
               es.tell(str_attackerid, '#multi', func_lang_text('pin pull', dict_tokens, playerlib.getPlayer(str_attackerid).get('lang')))
            if int(es.ServerVar('knifegrenade_announce_all')) == 2:
               for int_loop_userid in es.getUseridList():
                  es.cexec(int_loop_userid, 'echo ' + remove_tags(func_lang_text('alert all players', dict_tokens, playerlib.getPlayer(int_loop_userid).get('lang'))))
            elif int(es.ServerVar('knifegrenade_announce_all')):
               for int_loop_userid in es.getUseridList():
                  es.tell(int_loop_userid, '#multi', func_lang_text('alert all players', dict_tokens, playerlib.getPlayer(int_loop_userid).get('lang')))

            float_sound_percent = float(es.ServerVar('knifegrenade_sound_percentpinpull'))
            if float_sound_percent:
               es.emitsound('player', str_userid, 'weapons/pinpull.wav', 1.0, (100 - float_sound_percent) / 100)
            dict_players[str_userid] = {'count':0, 'attacker':str_attackerid}
            gamethread.delayedname(1, 'knifegrenade_%s' % str_userid, execute_tick, str_userid)


def player_death(event_var):
   """Detonates the player or removes the delay as necessary"""
   str_userid = event_var['userid']

   if dict_players.has_key(str_userid):
      if int(es.ServerVar('knifegrenade_pinpull_explodeondeath')) and event_var['weapon'] <> 'knife':
         gamethread.cancelDelayed('knifegrenade_%s' % str_userid)
         explode_player(str_userid)
      else:
         remove_delay(str_userid)


def player_disconnect(event_var):
   """Removes the delay for the disconnecting player"""
   remove_delay(event_var['userid'])


def weapon_fire(event_var):
   """Removes the delay if the thrown HE grenade had a pin pull delay"""
   if event_var['weapon'] == 'hegrenade':
      remove_delay(event_var['userid'])


def hegrenade_detonate(event_var):
   """Removes the delay in the detonated HE grenade had a pin pull delay (error check)"""
   remove_delay(event_var['userid'])


def unload():
   """
   Cancels outstanding delays
   Removes eventscripts_noisy
   """
   for str_userid in dict_players:
      gamethread.cancelDelayed('knifegrenade_%s' % str_userid)

   if int_noisy:
      es.ServerVar('eventscripts_noisy').set(0)
   else:
      es.doblock('corelib/noisy_off')

   for str_option in dict_options:
      es.ServerVar(str_option).set(0)


def remove_delay(str_userid):
   """Removes the delay for the player if necessary"""
   global dict_players

   if dict_players.has_key(str_userid):
      gamethread.cancelDelayed('knifegrenade_%s' % str_userid)
      del dict_players[str_userid]


def explode_player(str_userid):
   """
   Removes the exploding player's HE grenade
   Plays explosion sound
   Detonates explosion at str_userid giving credit to str_attackerid
   Removes str_userid from dict_players
   """
   global dict_players

   int_player_handle = es.getplayerhandle(str_userid)
   for int_index in es.createentitylist('weapon_hegrenade'):
      if int_player_handle == es.getindexprop(int_index, 'CBaseEntity.m_hOwnerEntity'):
         es.server.cmd('es_xremove %s' % int_index)

   float_sound_percent = float(es.ServerVar('knifegrenade_sound_percentdetonate'))
   if float_sound_percent:
      es.emitsound('player', str_userid, 'weapons/hegrenade/explode%s.wav' % random.randint(3, 5), '1.0', (100 - float_sound_percent) / 100)

   for int_index in es.createentitylist('env_explosion'):
      es.server.cmd('es_xremove %s' % int_index)
   es.server.cmd('es_xgive %s env_explosion' % str_userid)
   es.server.cmd('es_xfire %s env_explosion addoutput \"imagnitude %s\"' % (str_userid, es.ServerVar('knifegrenade_detonate_magnitude')))
   es.server.cmd('es_xfire %s env_explosion addoutput \"iradiusoverride %s\"' % (str_userid, es.ServerVar('knifegrenade_detonate_radius')))
   es.setindexprop(es.ServerVar('eventscripts_lastgive'), 'CBaseEntity.m_hOwnerEntity', es.getplayerhandle(dict_players[str_userid]['attacker']))
   es.server.cmd('es_xfire %s env_explosion explode' % str_userid)
   es.server.cmd('es_xfire %s env_explosion kill' % str_userid)

   del dict_players[str_userid]


def execute_tick(str_userid):
   """Plays tick sound or detonates explosion"""
   global dict_players

   dict_players[str_userid]['count'] += 1
   if dict_players[str_userid]['count'] >= es.ServerVar('knifegrenade_detonate_delay'):
      explode_player(str_userid)
   else:
      float_sound_percent = float(es.ServerVar('knifegrenade_sound_percenttick'))
      if float_sound_percent:
         es.emitsound('player', str_userid, 'weapons/grenade/tick1.wav', 1.0, (100 - float_sound_percent) / 100)
      gamethread.delayedname(1, 'knifegrenade_%s' % str_userid, execute_tick, str_userid)


def remove_tags(str_text):
   """Removes #lightgreen, #green and #default tags from the supplied string"""
   return str_text.replace('#lightgreen', '').replace('#green', '').replace('#default', '')


def missing_lang(str_heading, dict_tokens, str_language):
   """Displays an error message because the languages .ini could not be found"""
   return 'KnifeGrenade: knifegrenade_languages.ini not found'